#include "stdafx.h"
#include "Element.h"

Element::Element(){
}


void Element::calculateArea()
{
	MatrixXd J(2,2);

	MatrixXd XY(2,3);
	XY << x1, x2, x3, 
		  y1, y2, y3;

	MatrixXd dNdx(3,2);
	dNdx << 1, 0,
		    0, 1,
		   -1,-1;

	J << XY * dNdx;
	area = abs(0.5 * J.determinant());

}

void Element::calculateAreaCentre()
{
	ecx = (x1 + x2 + x3)/3;
	ecy = (y1 + y2 + y3)/3;
	
	xl1 = x1 - ecx;
	xl2 = x2 - ecx;
	xl3 = x3 - ecx;
	yl1 = y1 - ecy;
	yl2 = y2 - ecy;
	yl3 = y3 - ecy;
}

double Element::getSecondAreaMomentX(double Ay)
{
	double zeta[2][3] = {{0.5, 0.5, 0.0}, {0.0, 0.5, 0.5}};	// integration points
	double weight[3] = {1.0/3, 1.0/3, 1.0/3};

	double elementIx = 0;

	for(int i=0; i<3; i++){
		double fact = weight[i] * area;
		elementIx += fact * pow(yOfZeta(zeta[0][i], zeta[1][i]), 2);
	}
	double steiner = 0;
	steiner = area * pow((ecy-Ay), 2);
	elementIx += steiner;

	return elementIx;
}

double Element::getSecondAreaMomentY(double Ax)
{
	int nIntP = 3;
	double zeta[2][3] = {{0.5, 0.5, 0.0}, {0.0, 0.5, 0.5}};	// integration points
	double weight[3] = {1.0/3, 1.0/3, 1.0/3};

	double elementIy = 0;

	for(int i=0; i<nIntP; i++){
		double fact = weight[i] * area;
		elementIy += fact * pow(xOfZeta(zeta[0][i], zeta[1][i]), 2);
	}
	double steiner = 0;
	steiner = area * pow((ecx - Ax), 2);
	elementIy += steiner;

	return elementIy;
}


double Element::getSecondAreaMomentProduct(double Ax, double Ay)
{
	double zeta[2][3] = {{0.5, 0.5, 0.0}, {0.0, 0.5, 0.5}};
	double weight[3] = {1.0/3, 1.0/3, 1.0/3};

	double elementIProd = 0;
	for(int i=0; i<3; i++){
		double fact = weight[i] * area;
		elementIProd += fact * xOfZeta(zeta[0][i], zeta[1][i]) * yOfZeta(zeta[0][i], zeta[1][i]);
	}
	double steiner = 0;
	steiner = area * (ecx - Ax) * (ecy - Ay);
	elementIProd += steiner;

	return elementIProd;
}

MatrixXd Element::getShapefunction(double zeta1, double zeta2, double zeta3){
	MatrixXd shapeFunction(1, 3);
	shapeFunction << zeta1, zeta2, zeta3;
	return shapeFunction;
}

MatrixXd Element::getShapefunction(double x, double y){
	MatrixXd shapeFunction(1, 3);
	shapeFunction << (y2-y3)*x + (x3-x2)*y + (x2*y3-x3*y2), 
					(y3-y1)*x + (x1-x3)*y + (x3*y1-x1*y3), 
					(y1-y2)*x + (x2-x1)*y + (x1*y2-x2*y1);
	shapeFunction *= 0.5/area;
	return shapeFunction;
}


MatrixXd Element::getNMatrix(double zeta1, double zeta2, double zeta3){
	MatrixXd nMatrix(2, 6);
	MatrixXd null(1, 3);
	null << 0, 0, 0;
	nMatrix << null, -getShapefunction(zeta1, zeta2, zeta3), getShapefunction(zeta1, zeta2, zeta3), null;
	return nMatrix;
}

MatrixXd Element::getNMatrix(double x, double y){
	MatrixXd nMatrix(2, 6);
	MatrixXd null(1, 3);
	null << 0, 0, 0;
	nMatrix << null, -getShapefunction(x, y), getShapefunction(x, y), null;
	return nMatrix;
}

void Element::calculateConstants(){
	shearModule = 5;

	B.resize(2, 3);
	B << (y2-y3), (y3-y1), (y1-y2), 
		 (x3-x2), (x1-x3), (x2-x1);
	B *= (0.5/area);

	G.resize(2, 2);
	G << shearModule, 0, 0, shearModule;

	XoverY.resize(6, 1);
	XoverY << x1, x2, x3, y1, y2, y3;

	xCoordinates.resize(1, 3);
	yCoordinates.resize(1, 3);
	xCoordinates << x1, x2, x3;
	yCoordinates << y1, y2, y3;
}

std::vector<int> Element::getIEG()
{
	std::vector<int> IEG;

	IEG.push_back(i);
	IEG.push_back(j);
	IEG.push_back(k);

	return IEG;
}

double Element::xOfZeta(double zeta1, double zeta2){
	return (xl1-xl3)*zeta1+(xl2-xl3)*zeta2+xl3;
}

double Element::yOfZeta(double zeta1, double zeta2){
	return (yl1-yl3)*zeta1+(yl2-yl3)*zeta2+yl3;
}

void Element::calculateStiffnessMatrix(){
	elementStiffness.resize(2, 2);
	elementStiffness = area * B.transpose() * G * B;
}

void Element::calculateElementLoadTorsion(){

	double zeta[2][3] = {{0.5, 0.5, 0.0}, {0.0, 0.5, 0.5}};
	double weight[3] = {1.0/3, 1.0/3, 1.0/3};

	double zeta2[2][4] = {{1.0/3, 3.0/5, 1.0/5, 1.0/5}, {1.0/3, 1.0/5, 3.0/5, 1.0/5}};
	double weight2[4] = {-0.5625, 0.52083333333, 0.52083333333, 0.52083333333};

	MatrixXd strainI(2, 1);
	strainI << 0, 0;

	for(int i=0; i<3; i++){
		Vector2d init = getNMatrix(zeta[0][i], zeta[1][i], 1-zeta[0][i]-zeta[1][i]) * XoverY;
		strainI += weight[i] * init;
		initialStrain(0, i)=init(0);
		initialStrain(1, i)=init(1);
	}

	elementLoad.resize(3, 1);
	elementLoad = area * (B.transpose() * G * strainI);
}

void Element::calculateElementLoadShearX(){

	Vector2d epsilon;
	epsilon << 1, 0;

	elementLoad.resize(3, 1);
	elementLoad = area * (B.transpose() * G * epsilon);
	for(int i=0; i<3; i++){
		//initialStrain.push_back(epsilon);
		initialStrain(0, i)=epsilon(0);
		initialStrain(1, i)=epsilon(1);
	}
}

void Element::calculateElementLoadShearY(){

	Vector2d epsilon;
	epsilon << 0, -1;

	elementLoad.resize(3, 1);
	elementLoad = area * (B.transpose() * G * epsilon);
	for(int i=0; i<3; i++){
		//initialStrain.push_back(epsilon);
		initialStrain(0, i)=epsilon(0);
		initialStrain(1, i)=epsilon(1);
	}
}

void Element::setElementValue(MatrixXd &system, MatrixXd &local){
	std::vector<int> IEG = getIEG();
	local.resize(IEG.size(), 1);

	for(unsigned i=0; i<IEG.size(); i++){
		local(i, 0) = system(IEG[i], 0);
	}
}


void Element::computeTau(){


	for(int i=0; i<3; i++){
		Vector2d initStr;
		initStr << initialStrain(0, i), initialStrain(1, i);
		MatrixXd taui(2, 1);
		taui = (B * elementDisplacement + initStr);
		tau.push_back(taui);
	}
}

