#include "stdafx.h"
#include "Solver.h"

using namespace Eigen;

void getProgramArguments(int, char *[]);

std::string fileName;
double material;

int main(int argc, char *argv[])
{
	//Default values
	fileName = "ExampleMesh/testbeam";
	material = 200000;

	getProgramArguments(argc, argv);
	
	Solver solver(fileName, material);

	return 0;
}

void getProgramArguments(int argc, char *argv[])
{
	if(argc > 1)
		{
		for (int i = 0; i < argc; i++)
		{
			std::string arg = argv[i];
			if(arg == "-f")
			{
				fileName = argv[i+1];
			}
			if(arg == "-e")
			{
				material = atof(argv[i+1]);
			}
		}
		std::cout << "FileName: " << fileName << ", " << "E(Mpa): " << material << std::endl;
	}
}

Solver::Solver(std::string fileName, double material)
{
	Solver::fileName = fileName;
	MSHParser *mshParser = new MSHParser(fileName+".msh");
	elements = mshParser->getElements();
	nodes = mshParser->getNodes();

	nElements = mshParser->getNumberOfElements();
	nNodes = mshParser->getNumberOfNodes();

	mesh = Mesh(elements, nElements, nodes, nNodes, material);
	//mesh.setEModule(material);


	//std::cout << std::endl << "Start solve:" << std::endl;
	//mesh.solveForDisplacements();
	//std::cout << std::endl << "End solve" << std::endl;

	////Print results and save result file------
	printResults(mesh);
	saveResults(mesh);
} 

void Solver::saveResults(Mesh mesh)
{
	ResultWriter file(fileName+".res", mesh);
	file.writeMeshAreaProperties();
	file.writeNormalStress();
	file.writeMomentStress(nodes, nNodes);
	file.writeInertialAxes();
	file.save();

	ResultWriter VTK(fileName+".vtk", mesh);
	VTK.writeVtkHeader();
	VTK.writeMeshTopology(nNodes, nodes, nElements, elements);
	VTK.writePointScalarData("Sigma_due_to_N", mesh.sigmaAxial);
	VTK.writePointScalarData("Sigma_due_to_Mx", mesh.sigmaOfX);
	VTK.writePointScalarData("Sigma_due_to_My", mesh.sigmaOfY);
	VTK.writePointScalarData("tauXZtorsion", mesh.tauXZtorsion);
	VTK.writePointScalarData("tauYZtorsion", mesh.tauYZtorsion);
	VTK.writePointScalarData("tauRtorsion", mesh.tauRtorsion);
	/*VTK.writePointScalarData("tauXZshearX", mesh.tauXZshearX);
	VTK.writePointScalarData("tauYZshearX", mesh.tauYZshearX);
	VTK.writePointScalarData("tauRshearX", mesh.tauRshearX);
	VTK.writePointScalarData("tauXZshearY", mesh.tauXZshearY);
	VTK.writePointScalarData("tauYZshearY", mesh.tauYZshearY);
	VTK.writePointScalarData("tauRshearY", mesh.tauRshearY);*/

	VTK.writePointVectorData("tauXZtorsion_vec", mesh.tauXZtorsion);
	VTK.writePointVectorData("tauYZtorsion_vec", mesh.tauYZtorsion);
	VTK.writePointVectorData("tauRtorsion_vec", mesh.tauRtorsion);
	/*VTK.writePointVectorData("tauXZshearX_vec", mesh.tauXZshearX);
	VTK.writePointVectorData("tauYZshearX_vec", mesh.tauYZshearX);
	VTK.writePointVectorData("tauRshearX_vec", mesh.tauRshearX);
	VTK.writePointVectorData("tauXZshearY_vec", mesh.tauXZshearY);
	VTK.writePointVectorData("tauYZshearY_vec", mesh.tauYZshearY);
	VTK.writePointVectorData("tauRshearY_vec", mesh.tauRshearY);
	VTK.writePointVectorData("PrintTemp1", mesh.printTemp1);*/
	VTK.writePointVectorData("PrintTemp2", mesh.printTemp2);

	VTK.save();
}

void Solver::printResults(Mesh mesh)
{
	double Ix = mesh.Ix;
	double Iy = mesh.Iy;
	double Ixy = mesh.Ixy;

//	double shearCentreX = mesh.computeShearXCentre();
//	double shearCentreY = mesh.computeShearYCentre();

	std::cout << std::endl << "RESULTS---------------------------------------------" << std::endl;

	std::cout << "AreaCentre: [" << mesh.Ax << "," << mesh.Ay << "]" << std::endl;
	std::cout << "TotalArea: " << mesh.totalArea << std::endl;

//	std::cout << "ShearCentre: " << "[" << shearCentreX << "," << shearCentreY << "]" << std::endl;
	std::cout << "SecondAreaMoment: [Ix,Iy,Ixy]=[" << Ix << ", " << Iy << ", " << Ixy <<"]" << std::endl;
	std::cout << "PrincipalAxes(Deg): [" << mesh.getAnglePrincipalAxes() << "]" << std::endl;


//	vector<int> constrained = mesh.getNodesNearCentre(2);
//	std::cout << std::endl << "Two-nodes close to Axy: [" << constrained[0] << "," << constrained[1] << "]" << std::endl;
}

