#include "stdafx.h"
#include "MSHParser.h"
#include "Node.h"
#include "Mesh.h"
#include "Element.h"
#include "ResultWriter.h"

#ifndef SOLVER
#define SOLVER

using namespace Eigen;

//! Contains the main function and initializes MSHParser, Mesh, and finally ResultWriter.
class Solver
{
public:
	Solver(std::string, double material);
	
private:
	std::string fileName;	//! < Filename excluding file ending.
	double material;		//! < Youngs modulus for the mesh material
	Mesh mesh;				//! < Reference to the mesh object containing the mesh data.
	int nElements;			//! < Number of elements in mesh
	int nNodes;				//! < Number of nodes in mesh

	std::vector<Node> nodes;		//! < Vector with all nodes in the mesh
	std::vector<Element> elements;	//! < Vector with all elements in the mesh

	/*!
		Opens the result file and saves results to file. 
		The results are written to file by using the ResultWriter.
	*/
	void saveResults(Mesh);
	/*!
		Prints the meshproperties to console for fast verification
	*/
	void printResults(Mesh);
};

#endif